SystemOrganization addCategory: #'Xpf-Server-Models'!
SystemOrganization addCategory: #'Xpf-Server-Rest'!

RestEntity subclass: #XpfEntity
	instanceVariableNames: 'revision isDeleted'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

!XpfEntity commentStamp: 'mt 2/12/2009 12:20' prior: 0!
I am the abstract superclass of all entities in the system. These entities could be user stories, tasks, developers, themes, ...!

!XpfEntity class methodsFor: 'accessing' stamp: 'topa 11/27/2009 17:52'!
collectionClass

	^ OrderedCollection! !

!XpfEntity class methodsFor: 'accessing' stamp: 'cs 5/19/2009 18:25'!
handlerClass

	^ XpfRestResourceHttpHandler! !

!XpfEntity class methodsFor: 'description' stamp: 'mt 5/7/2009 10:21'!
primitivePropertyNames

	^ super primitivePropertyNames, #(revision isDeleted updateInterval)! !

!XpfEntity methodsFor: 'deleting' stamp: 'cs 6/2/2009 16:06'!
delete

	self pvtIsDeleted: true.! !

!XpfEntity methodsFor: 'revisions' stamp: 'topa 4/2/2009 18:42'!
incrementRevision

	self pvtRevision: self revision + 1.! !

!XpfEntity methodsFor: 'testing' stamp: 'mt 4/21/2009 13:58'!
isDeleted

	^ self pvtIsDeleted! !

!XpfEntity methodsFor: 'deleting' stamp: 'mt 4/21/2009 14:17'!
isDeleted: aBoolean
	"Read-only."
	
	^ self! !

!XpfEntity methodsFor: 'authorization' stamp: 'topa 4/7/2009 14:08'!
isMethod: aMethodString allowedFor: aUser
	"returns whether the method is allowed for the specified user; may be overwritten in subclasses"

	^ true! !

!XpfEntity methodsFor: 'testing' stamp: 'topa 3/24/2009 17:27'!
isXpfEntity

	^ true! !

!XpfEntity methodsFor: 'private' stamp: 'mt 4/21/2009 13:55'!
pvtIsDeleted

	^ isDeleted ifNil: [false]! !

!XpfEntity methodsFor: 'private' stamp: 'mt 4/21/2009 13:55'!
pvtIsDeleted: aBoolean

	isDeleted := aBoolean.! !

!XpfEntity methodsFor: 'private' stamp: 'topa 4/2/2009 18:42'!
pvtRevision: aNumber

	revision := aNumber.! !

!XpfEntity methodsFor: 'printing' stamp: 'mt 5/6/2009 16:12'!
responseDepth: depth
	"Returns a Dictionary with the properties. If the properties are entities and the depth is higher than 1 then resolve recursivly."

	| result |
	result := super responseDepth: depth.
	depth = 0 ifTrue: [^ result].
	result addAll: (Dictionary newFrom: 
		(self class collectionPropertyNames collect: [:aSymbol | 
			self responseCollection: aSymbol, 'WithDeleted' depth: depth - 1])).
	^ result! !

!XpfEntity methodsFor: 'accessing' stamp: 'topa 4/2/2009 18:42'!
revision

	^ revision ifNil: [revision := 1]! !

!XpfEntity methodsFor: 'accessing' stamp: 'mt 4/6/2009 16:51'!
revision: anObject
	"rest and GemStone compatibility:"

	^ self! !

!XpfEntity methodsFor: 'updating' stamp: 'mt 5/7/2009 10:17'!
updateInterval
	"Tell the client how often a changes request for this entity should be sent."
	
	^ 0 "Never."! !

!XpfEntity methodsFor: 'updating' stamp: 'mt 5/7/2009 10:23'!
updateInterval: aNumber
	"readonly"
	
	^ self! !

XpfEntity subclass: #XpfEntityWithHistory
	instanceVariableNames: 'history'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

!XpfEntityWithHistory class methodsFor: 'description' stamp: 'cs 4/17/2009 11:56'!
collectionPropertyNames

	^ super collectionPropertyNames, #(history)! !

!XpfEntityWithHistory class methodsFor: 'description' stamp: 'cs 4/17/2009 11:48'!
historyActionNames

	^ #(created changed deleted added removed)! !

!XpfEntityWithHistory methodsFor: 'deleting' stamp: 'cs 4/17/2009 11:40'!
delete

	super delete.
	self historyAddEntry: #deleted.! !

!XpfEntityWithHistory methodsFor: 'history' stamp: 'cs 4/17/2009 11:36'!
history

	history ifNil: [history := OrderedCollection new].
	^ history! !

!XpfEntityWithHistory methodsFor: 'history' stamp: 'cs 4/17/2009 11:37'!
history: anObject
	"History is read-only."

	^ self
	! !

!XpfEntityWithHistory methodsFor: 'history' stamp: 'cs 4/17/2009 11:37'!
historyAddEntry: action

	self historyAddEntry: action for: nil.! !

!XpfEntityWithHistory methodsFor: 'history' stamp: 'cs 4/17/2009 11:37'!
historyAddEntry: action for: property

	self historyAddEntry: action for: property to: nil.! !

!XpfEntityWithHistory methodsFor: 'history' stamp: 'cs 4/17/2009 12:02'!
historyAddEntry: action for: property from: oldValue to: newValue
	"Add entry to entity's history."
	
	self history add:
		(XpfHistoryEntry
			entity: self
			action: action
			property: property
			from: oldValue
			to: newValue).
	self incrementRevision.! !

!XpfEntityWithHistory methodsFor: 'history' stamp: 'cs 4/17/2009 11:37'!
historyAddEntry: action for: property to: newValue

	self historyAddEntry: action for: property from: nil to: newValue.! !

!XpfEntityWithHistory methodsFor: 'history' stamp: 'cs 4/17/2009 12:08'!
historyWithDeleted

	^ self history! !

!XpfEntityWithHistory methodsFor: 'initialize-release' stamp: 'cs 4/17/2009 11:40'!
initialize

	super initialize.
	self historyAddEntry: #created.! !

XpfEntityWithHistory subclass: #XpfIteration
	instanceVariableNames: 'release title userStories timeIdeal'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

XpfIteration subclass: #XpfFutureIteration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

!XpfFutureIteration commentStamp: 'mt 4/15/2010 15:04:23.107' prior: 0!
I am a container for open stories used in releases.!

!XpfFutureIteration class methodsFor: 'description' stamp: 'mt 4/15/2010 15:02:59.758'!
primitivePropertyNames

	^ super primitivePropertyNames, #(isForProject)! !

!XpfFutureIteration methodsFor: 'accessing' stamp: 'topa 3/24/2009 17:18'!
id

	^ -1! !

!XpfFutureIteration methodsFor: 'testing' stamp: 'mt 4/15/2010 15:02:24.11'!
isForProject

	^ false! !

!XpfFutureIteration methodsFor: 'testing' stamp: 'topa 3/24/2009 17:18'!
isXpfFutureIteration

	^ true! !

!XpfFutureIteration methodsFor: 'as yet unclassified' stamp: 'topa 4/7/2009 14:12'!
moveNilUserStoriesToFuture
	
	self project userStories do: [:us | 
		us iteration ifNil: [self addUserStory: us]].! !

!XpfFutureIteration methodsFor: 'as yet unclassified' stamp: 'SR 4/12/2009 21:15'!
timeIdeal

	^ 0! !

!XpfFutureIteration methodsFor: 'as yet unclassified' stamp: 'SR 4/12/2009 21:15'!
timeIdeal: anObject

	^ self! !

!XpfFutureIteration methodsFor: 'accessing' stamp: 'mt 4/16/2010 15:11:25.405'!
title

	^ 'Open Stories (R)'! !

!XpfFutureIteration methodsFor: 'accessing' stamp: 'topa 4/7/2009 14:14'!
title: aString
	"rest and GemStone compatibility"
	
	^ self! !

XpfFutureIteration subclass: #XpfFutureIterationForProject
	instanceVariableNames: 'project'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

!XpfFutureIterationForProject commentStamp: 'mt 4/15/2010 15:00:25.057' prior: 0!
I am a special future iteration (= container for open stories) for a whole project. Do not try to ask for my #theRelease. ;)!

!XpfFutureIterationForProject class methodsFor: 'description' stamp: 'mt 4/15/2010 15:06:33.887'!
referencePropertyNames

	^ super referencePropertyNames reject: [:sym | sym = #theRelease]! !

!XpfFutureIterationForProject methodsFor: 'testing' stamp: 'mt 4/15/2010 15:02:39.028'!
isForProject

	^ true! !

!XpfFutureIterationForProject methodsFor: 'accessing' stamp: 'mt 4/15/2010 15:00:50.216'!
project

	^ project! !

!XpfFutureIterationForProject methodsFor: 'accessing' stamp: 'mt 4/15/2010 15:01:03.986'!
project: anXpfProject

	self incrementRevision.
	project := anXpfProject.! !

!XpfFutureIterationForProject methodsFor: 'accessing' stamp: 'mt 4/16/2010 15:11:30.605'!
title

	^ 'Open Stories (P)'! !

!XpfIteration class methodsFor: 'description' stamp: 'SR 4/7/2009 11:33'!
collectionPropertyNames

	^ super collectionPropertyNames, #(userStories)! !

!XpfIteration class methodsFor: 'instance creation' stamp: 'SR 3/25/2009 18:13'!
newFrom: aDictionary

	^ self new title: (aDictionary at: 'title' ifAbsent: [])! !

!XpfIteration class methodsFor: 'description' stamp: 'SR 4/12/2009 21:15'!
primitivePropertyNames

	^ super primitivePropertyNames, #(title isXpfFutureIteration timeIdeal)! !

!XpfIteration class methodsFor: 'description' stamp: 'mt 4/15/2010 13:39:51.627'!
referencePropertyNames

	^ super referencePropertyNames, #(project theRelease)! !

!XpfIteration methodsFor: 'add-remove' stamp: 'SR 3/25/2009 18:15'!
addNewUserStory: aDictionary

	^ self addUserStory: (XpfUserStory newFrom: aDictionary)! !

!XpfIteration methodsFor: 'add-remove' stamp: 'cs 6/2/2009 13:31'!
addUserStory: aUserStory

	| oldTaskRestUris oldUserStoryRestUri |
	aUserStory id ifNil: [aUserStory id: aUserStory class incrementId].
	aUserStory iteration ifNotNil:
		[oldUserStoryRestUri := aUserStory restUri.
		oldTaskRestUris := aUserStory tasks collect: [:task | task restUri].
		aUserStory iteration removeUserStory: aUserStory].
	aUserStory
		iteration: self;
		handler: self handler.
	(self pvtUserStories includes: aUserStory) ifFalse: 
		[self pvtUserStories add: aUserStory.
		self 
			historyAddEntry: #added
			for: #userStories
			to: aUserStory.
		oldUserStoryRestUri ifNotNil: 
			[XpfRestResourceHttpHandler 
				redirectRestUriFrom: oldUserStoryRestUri
				to: aUserStory restUri.
			oldTaskRestUris withIndexDo: 
				[:uri :index | 
				XpfRestResourceHttpHandler 
					redirectRestUriFrom: uri
					to: (aUserStory tasks at: index) restUri]]].
	^ aUserStory! !

!XpfIteration methodsFor: 'deleting' stamp: 'mt 4/15/2010 13:49:34.086'!
delete

	self userStories do: [:us | us delete].
	self theRelease historyAddEntry: #removed for: #iterations to: self.
	super delete.
	! !

!XpfIteration methodsFor: 'initialization' stamp: 'mt 4/16/2009 11:09'!
initialize

	super initialize.
	self pvtUserStories: OrderedCollection new.! !

!XpfIteration methodsFor: 'accessing' stamp: 'SR 4/7/2009 11:15'!
isXpfFutureIteration

	^ false! !

!XpfIteration methodsFor: 'accessing' stamp: 'topa 4/7/2009 14:11'!
isXpfFutureIteration: aBoolean
	"rest and GemStone compatibility"
	
	^ self! !

!XpfIteration methodsFor: 'printing' stamp: 'SR 3/20/2009 20:56'!
printString

	^ self title ifNil: ['No Title']! !

!XpfIteration methodsFor: 'accessing-convenience' stamp: 'mt 4/15/2010 13:38:37.635'!
project
	"This is just a convenient accessor."
	
	^ self theRelease ifNotNilDo: [:release |
		release project].! !

!XpfIteration methodsFor: 'accessing-convenience' stamp: 'mt 4/26/2010 12:07:48.348'!
project: anObject
	"Deprecated. Do nothing. Use the release object."
	
	self deprecated: '[Xpf-Server] Warning: XpfIteration>>project: should not be used.'.
	
	^ self! !

!XpfIteration methodsFor: 'accessing' stamp: 'mt 4/16/2009 10:59'!
pvtUserStories

	^ userStories! !

!XpfIteration methodsFor: 'accessing' stamp: 'mt 4/16/2009 10:59'!
pvtUserStories: anObject

	userStories := anObject.! !

!XpfIteration methodsFor: 'private' stamp: 'cs 5/20/2009 14:03'!
removeUserStory: aUserStory 

	(self userStories noneSatisfy: [:us | us = aUserStory]) ifTrue: [^ aUserStory].
	self pvtUserStories remove: aUserStory.
	self 
		historyAddEntry: #removed
		for: #userStories
		to: aUserStory.
	aUserStory iteration: nil.
	^ aUserStory! !

!XpfIteration methodsFor: 'accessing' stamp: 'mt 4/15/2010 13:36:23.502'!
theRelease

	^ release! !

!XpfIteration methodsFor: 'accessing' stamp: 'mt 4/15/2010 13:36:54.624'!
theRelease: anXpfRelease

	self incrementRevision.
	release := anXpfRelease.! !

!XpfIteration methodsFor: 'accessing' stamp: 'SR 4/12/2009 21:15'!
timeIdeal

	^ timeIdeal! !

!XpfIteration methodsFor: 'accessing' stamp: 'SR 4/12/2009 21:15'!
timeIdeal: anObject

	timeIdeal := anObject! !

!XpfIteration methodsFor: 'accessing' stamp: 'topa 3/24/2009 17:21'!
title

	^ title! !

!XpfIteration methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:26'!
title: anObject

	anObject = self title ifTrue: [^ self].
	self historyAddEntry: #changed for: #title from: self title to: anObject.
	title := anObject.! !

!XpfIteration methodsFor: 'updating' stamp: 'mt 4/15/2010 13:53:08.598'!
updateInterval

	^ 3! !

!XpfIteration methodsFor: 'accessing' stamp: 'mt 4/16/2009 10:59'!
userStories

	^ userStories reject: [:us | us isDeleted]! !

!XpfIteration methodsFor: 'accessing' stamp: 'mt 5/6/2009 13:05'!
userStoriesWithDeleted

	^ userStories! !

XpfEntityWithHistory subclass: #XpfProject
	instanceVariableNames: 'title developers releases events futureIteration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

!XpfProject class methodsFor: 'description' stamp: 'mt 4/15/2010 13:55:07.284'!
collectionPropertyNames

	^ super collectionPropertyNames, #(developers releases events)! !

!XpfProject class methodsFor: 'instance creation' stamp: 'SR 3/25/2009 18:08'!
newFrom: aDictionary

	^ self new title: (aDictionary at: 'title')! !

!XpfProject class methodsFor: 'description' stamp: 'topa 3/3/2009 22:26'!
primitivePropertyNames

	^ super primitivePropertyNames, #(title)! !

!XpfProject class methodsFor: 'description' stamp: 'mt 4/15/2010 14:57:59.052'!
referencePropertyNames

	^ super referencePropertyNames, #(futureIteration)! !

!XpfProject methodsFor: 'add-remove' stamp: 'cs 5/20/2009 14:05'!
addDeveloper: aUser

	self pvtDevelopers 
		add: aUser
		ifPresent: [^ aUser].
	aUser addProject: self.
	self 
		historyAddEntry: #added
		for: #developers
		to: aUser.
	^ aUser! !

!XpfProject methodsFor: 'add-remove' stamp: 'FHH 11/19/2009 18:48'!
addEvent: anEvent
	self removeOldEvents.
	(self events includes: anEvent) ifTrue: [^ anEvent].
	anEvent
		id: XpfEvent incrementId;
		project: self;
		handler: self handler.
	^ self events add: anEvent! !

!XpfProject methodsFor: 'add-remove' stamp: 'FHH 7/9/2009 14:37'!
addNewEvent: aDictionary

	^ self addEvent: (XpfEvent newFrom: aDictionary)! !

!XpfProject methodsFor: 'add-remove' stamp: 'mt 4/15/2010 13:57:53.123'!
addNewRelease: aDictionary

	^ self addRelease: (XpfRelease newFrom: aDictionary)! !

!XpfProject methodsFor: 'add-remove' stamp: 'mt 4/15/2010 13:57:36.174'!
addRelease: anXpfRelease

	(self pvtReleases includes: anXpfRelease) 
		ifTrue: [^ anXpfRelease].
	self pvtReleases add: anXpfRelease.
	anXpfRelease
		handler: self handler;
		project: self;
		id: anXpfRelease class incrementId.
	self historyAddEntry: #added for: #releases to: anXpfRelease.
	^ anXpfRelease! !

!XpfProject methodsFor: 'deleting' stamp: 'mt 4/15/2010 13:59:02.573'!
delete
	
	self releases do: [:rel | rel delete].
	super delete.! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 4/16/2009 10:41'!
developers

	^ developers reject: [:dev | dev isDeleted]! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 5/6/2009 13:05'!
developersWithDeleted

	^ developers! !

!XpfProject methodsFor: 'accessing' stamp: 'FHH 7/8/2009 17:07'!
events

	^ self eventsWithDeleted.! !

!XpfProject methodsFor: 'accessing' stamp: 'FHH 7/8/2009 17:07'!
eventsWithDeleted

	^ events ifNil: [events := OrderedCollection new.].! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 4/15/2010 14:57:34.554'!
futureIteration

	^ futureIteration.! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 4/15/2010 14:57:20.477'!
futureIteration: anXpfFutureIteration

	self incrementRevision.
	futureIteration := anXpfFutureIteration.! !

!XpfProject methodsFor: 'initialization' stamp: 'mt 4/15/2010 15:08:41.142'!
initialize

	super initialize.
	self
		pvtReleases: OrderedCollection new;
		pvtDevelopers: OrderedCollection new.
		
	futureIteration := XpfFutureIterationForProject new project: self.
! !

!XpfProject methodsFor: 'authorization' stamp: 'cs 3/18/2009 15:28'!
isMethod: aMethodString allowedFor: aUser

	^ self developers includes: aUser! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 4/15/2010 15:41:44.809'!
openStories
	"Used for resolving and creating a REST-conform URI. See XpfRestResourceHttpHandler>>uriMappings."

	^ OrderedCollection with: futureIteration.! !

!XpfProject methodsFor: 'printing' stamp: 'mt 2/25/2009 10:16'!
printString

	^ self title ifNil: ['No Title']! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 4/16/2009 10:41'!
pvtDevelopers

	^ developers! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 4/16/2009 10:41'!
pvtDevelopers: anObject

	developers := anObject.! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 4/15/2010 13:53:53.265'!
pvtReleases

	^ releases! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 4/15/2010 13:54:04.774'!
pvtReleases: anObject

	releases := anObject.! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 4/15/2010 13:54:33.772'!
releases

	^ self pvtReleases reject: [:rel | rel isDeleted]! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 4/15/2010 13:54:45.03'!
releasesWithDeleted

	^ self pvtReleases! !

!XpfProject methodsFor: 'add-remove' stamp: 'cs 5/20/2009 14:05'!
removeDeveloper: aUser

	self pvtDevelopers 
		remove: aUser
		ifAbsent: [^ aUser].
	self 
		historyAddEntry: #removed
		for: #developers
		to: aUser.
	aUser removeProject: self.
	^ aUser! !

!XpfProject methodsFor: 'add-remove' stamp: 'FHH 7/8/2009 17:02'!
removeEvent: anXpaEvent
	^ self events remove: anXpaEvent! !

!XpfProject methodsFor: 'add-remove' stamp: 'FHH 11/19/2009 18:48'!
removeOldEvents
	self events removeAllSuchThat: [ :event |
		(event details at: 'dateAndTime') asDateAndTime <= (DateAndTime now - (Duration weeks: 4)).
	].! !

!XpfProject methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:24'!
title

	^ title! !

!XpfProject methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:26'!
title: anObject

	anObject = self title ifTrue: [^ self].
	self historyAddEntry: #changed for: #title from: self title to: anObject.
	title := anObject.! !

!XpfProject methodsFor: 'updating' stamp: 'mt 4/15/2010 13:53:19.855'!
updateInterval
	"Tell the client how often a changes request for this entity should be sent."
	
	^ 3! !

XpfEntityWithHistory subclass: #XpfRelease
	instanceVariableNames: 'title iterations project'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

!XpfRelease class methodsFor: 'description' stamp: 'mt 4/15/2010 12:58:44.085'!
collectionPropertyNames

	^ super collectionPropertyNames, #(iterations)! !

!XpfRelease class methodsFor: 'instance creation' stamp: 'mt 4/15/2010 14:01:49.473'!
newFrom: aDictionary

	^ self new title: (aDictionary at: 'title' ifAbsent: [])! !

!XpfRelease class methodsFor: 'description' stamp: 'mt 4/15/2010 12:58:51.874'!
primitivePropertyNames

	^ super primitivePropertyNames, #(title)! !

!XpfRelease class methodsFor: 'description' stamp: 'mt 4/15/2010 13:44:36.581'!
referencePropertyNames

	^ super referencePropertyNames, #(project)! !

!XpfRelease methodsFor: 'add-remove' stamp: 'mt 4/15/2010 13:46:41.489'!
addIteration: anXpfIteration

	(anXpfIteration isXpfFutureIteration or: [self pvtIterations includes: anXpfIteration]) 
		ifTrue: [^ anXpfIteration].
	self pvtIterations add: anXpfIteration before: (self pvtIterations last).
	anXpfIteration
		handler: self handler;
		theRelease: self;
		id: anXpfIteration class incrementId.
	self historyAddEntry: #added for: #iterations to: anXpfIteration.
	^ anXpfIteration! !

!XpfRelease methodsFor: 'add-remove' stamp: 'mt 4/15/2010 13:47:15.305'!
addNewIteration: aDictionary

	^ self addIteration: (XpfIteration newFrom: aDictionary)! !

!XpfRelease methodsFor: 'deleting' stamp: 'mt 4/15/2010 13:49:18.237'!
delete
	
	self iterations do: [:iter | iter delete].
	self project historyAddEntry: #removed for: #releases to: self.
	super delete.! !

!XpfRelease methodsFor: 'initialization' stamp: 'mt 4/15/2010 14:52:57.703'!
initialize

	super initialize.
	self pvtIterations: (OrderedCollection with: (XpfFutureIteration new theRelease: self)).! !

!XpfRelease methodsFor: 'accessing' stamp: 'mt 4/15/2010 13:26:38.512'!
iterations

	^ self pvtIterations reject: [:it | it isDeleted]! !

!XpfRelease methodsFor: 'accessing' stamp: 'mt 4/15/2010 13:27:21.567'!
iterationsWithDeleted

	^ self pvtIterations! !

!XpfRelease methodsFor: 'printing' stamp: 'mt 4/15/2010 13:51:28.282'!
printString

	^ self title ifNil: ['No Title']! !

!XpfRelease methodsFor: 'accessing' stamp: 'mt 4/15/2010 13:43:44.517'!
project

	^ project! !

!XpfRelease methodsFor: 'accessing' stamp: 'mt 4/15/2010 13:44:09.744'!
project: anXpfProject

	self incrementRevision.
	project := anXpfProject.! !

!XpfRelease methodsFor: 'accessing' stamp: 'mt 4/15/2010 13:26:58.854'!
pvtIterations

	^ iterations! !

!XpfRelease methodsFor: 'accessing' stamp: 'mt 4/15/2010 13:27:05.979'!
pvtIterations: anObject

	iterations := anObject.! !

!XpfRelease methodsFor: 'accessing' stamp: 'mt 4/15/2010 13:25:18.745'!
title

	^ title! !

!XpfRelease methodsFor: 'accessing' stamp: 'mt 4/15/2010 13:25:55.886'!
title: aString

	aString = self title ifTrue: [^ self].
	self historyAddEntry: #changed for: #title from: self title to: aString.
	title := aString.! !

!XpfRelease methodsFor: 'updating' stamp: 'mt 4/15/2010 14:00:08.97'!
updateInterval

	^ 3! !

XpfEntityWithHistory subclass: #XpfTask
	instanceVariableNames: 'title userStory timeEstimated timeActual developers isDone responsibleDeveloper isStarted lastStarted startTimes stopTimes tests'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

!XpfTask commentStamp: 'mt 2/12/2009 12:22' prior: 0!
I am a task in tems of Extreme Programming.!

!XpfTask class methodsFor: 'description' stamp: 'mt 4/15/2009 17:10'!
collectionPropertyNames

	^ super collectionPropertyNames, #(developers)! !

!XpfTask class methodsFor: 'instance creation' stamp: 'mt 4/15/2009 16:58'!
newFrom: aDictionary

	^ self new
		title: (aDictionary at: 'title' ifAbsent: []);
		timeActual: (aDictionary at: 'timeActual' ifAbsent: []);
		timeEstimated: (aDictionary at: 'timeEstimated' ifAbsent: []);
		isDone: (aDictionary at: 'isDone' ifAbsent: [false])! !

!XpfTask class methodsFor: 'description' stamp: 'topa 11/27/2009 17:51'!
primitivePropertyNames

	^ super primitivePropertyNames, #(title timeEstimated timeActual isDone isStarted startTimes stopTimes tests)! !

!XpfTask class methodsFor: 'description' stamp: 'mt 4/16/2009 13:31'!
referencePropertyNames

	^ super referencePropertyNames, #(responsibleDeveloper userStory)! !

!XpfTask methodsFor: 'add-remove' stamp: 'cs 5/20/2009 14:05'!
addDeveloper: aUser

	self pvtDevelopers 
		add: aUser
		ifPresent: [^ aUser].
	aUser addTask: self.
	self 
		historyAddEntry: #added
		for: #developers
		to: aUser.
	^ aUser! !

!XpfTask methodsFor: 'deleting' stamp: 'mt 4/16/2009 12:56'!
delete

	self userStory historyAddEntry: #removed for: #tasks to: self.
	super delete.! !

!XpfTask methodsFor: 'accessing' stamp: 'mt 4/16/2009 11:19'!
developers

	^ developers reject: [:dev | dev isDeleted]! !

!XpfTask methodsFor: 'accessing' stamp: 'mt 5/6/2009 13:05'!
developersWithDeleted

	^ developers! !

!XpfTask methodsFor: 'initialization' stamp: 'topa 11/27/2009 18:08'!
initialize

	super initialize.
	self 
		pvtDevelopers: OrderedCollection new;
		tests: self class collectionClass new.! !

!XpfTask methodsFor: 'accessing' stamp: 'test 4/16/2009 13:36'!
isDone

	^ isDone ifNil: [isDone := false]! !

!XpfTask methodsFor: 'accessing' stamp: 'cs 4/23/2009 14:12'!
isDone: anObject

	anObject = self isDone ifTrue: [^ self].
	self isDone & self isStarted ifTrue: [self isStarted: false].
	self historyAddEntry: #changed for: #isDone from: self isDone to: anObject.
	isDone := anObject.! !

!XpfTask methodsFor: 'accessing' stamp: 'cs 4/23/2009 13:41'!
isStarted

	^ isStarted ifNil: [isStarted := false]! !

!XpfTask methodsFor: 'accessing' stamp: 'cs 4/23/2009 14:08'!
isStarted: anObject

	anObject = self isStarted ifTrue: [^ self].
	(self isStarted not & anObject) ifTrue: [self startWork].
	(self isStarted & anObject not) ifTrue: [self stopWork].
	self historyAddEntry: #changed for: #isStarted from: self isStarted to: anObject.
	isStarted := anObject.! !

!XpfTask methodsFor: 'accessing' stamp: 'cs 4/23/2009 13:49'!
lastStarted

	^ lastStarted! !

!XpfTask methodsFor: 'accessing' stamp: 'cs 4/23/2009 13:49'!
lastStarted: anObject

	lastStarted := anObject.! !

!XpfTask methodsFor: 'accessing' stamp: 'mt 4/16/2009 11:18'!
pvtDevelopers

	^ developers! !

!XpfTask methodsFor: 'accessing' stamp: 'mt 4/16/2009 11:18'!
pvtDevelopers: anObject

	developers := anObject.! !

!XpfTask methodsFor: 'add-remove' stamp: 'cs 5/20/2009 14:06'!
removeDeveloper: aUser

	self pvtDevelopers 
		remove: aUser
		ifAbsent: [^ aUser].
	self 
		historyAddEntry: #removed
		for: #developers
		to: aUser.
	aUser removeTask: self.
	^ aUser! !

!XpfTask methodsFor: 'accessing' stamp: 'SR 4/10/2009 15:07'!
responsibleDeveloper

	^ responsibleDeveloper! !

!XpfTask methodsFor: 'accessing' stamp: 'cs 5/20/2009 14:06'!
responsibleDeveloper: anObject

	anObject = self responsibleDeveloper ifTrue: [^ self].
	self 
		historyAddEntry: #changed
		for: #responsibleDeveloper
		from: self responsibleDeveloper
		to: anObject.
	responsibleDeveloper := anObject.
	self addDeveloper: anObject.! !

!XpfTask methodsFor: 'accessing' stamp: 'FHH 7/8/2009 14:21'!
startTimes
	^ startTimes ifNil: [ startTimes := SortedCollection new. ].! !

!XpfTask methodsFor: 'accessing' stamp: 'FHH 7/8/2009 14:19'!
startWork
	self timeActual ifNil: [self timeActual: 0].
	self updateStartTimesWith: DateAndTime now.! !

!XpfTask methodsFor: 'accessing' stamp: 'FHH 7/8/2009 14:21'!
stopTimes
	^ stopTimes ifNil: [ stopTimes := SortedCollection new. ].! !

!XpfTask methodsFor: 'accessing' stamp: 'FHH 7/8/2009 14:22'!
stopWork
	self updateStopTimesWith: DateAndTime now.! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 11/27/2009 18:08'!
tests

	^ tests! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 11/27/2009 17:53'!
tests: anObject

	anObject = self tests ifTrue: [^ self].
	self historyAddEntry: #changed for: #tests from: self tests to: anObject.
	tests := anObject.! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:31'!
timeActual

	^ timeActual! !

!XpfTask methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:26'!
timeActual: anObject

	anObject = self timeActual ifTrue: [^ self].
	self historyAddEntry: #changed for: #timeActual from: self timeActual to: anObject.
	timeActual := anObject.! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:31'!
timeEstimated

	^ timeEstimated! !

!XpfTask methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:27'!
timeEstimated: anObject

	anObject = self timeEstimated ifTrue: [^ self].
	self historyAddEntry: #changed for: #timeEstimated from: self timeEstimated to: anObject.
	timeEstimated := anObject.! !

!XpfTask methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:31'!
title

	^ title! !

!XpfTask methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:27'!
title: anObject

	anObject = self title ifTrue: [^ self].
	self historyAddEntry: #changed for: #title from: self title to: anObject.
	title := anObject.! !

!XpfTask methodsFor: 'updating' stamp: 'mt 5/7/2009 10:28'!
updateInterval
	"Tell the client how often a changes request for this entity should be sent."
	
	^ 3! !

!XpfTask methodsFor: 'accessing' stamp: 'FHH 7/8/2009 14:21'!
updateStartTimesWith: aDateAndTime
	self startTimes add: aDateAndTime.
	self lastStarted: aDateAndTime.! !

!XpfTask methodsFor: 'accessing' stamp: 'FHH 7/8/2009 14:23'!
updateStopTimesWith: aDateAndTime
	| delta |
	self stopTimes add: aDateAndTime.
	delta := Timespan starting: self lastStarted ending: aDateAndTime.
	self timeActual: self timeActual + (delta duration asSeconds / 3600) asFloat.! !

!XpfTask methodsFor: 'accessing' stamp: 'mt 4/15/2009 16:59'!
userStory

	^ userStory! !

!XpfTask methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:27'!
userStory: anXpfUserStory

	self userStory = anXpfUserStory ifTrue: [^ anXpfUserStory].
	userStory := anXpfUserStory.
	self historyAddEntry: #changed for: #userStory from: self userStory to: anXpfUserStory.! !

XpfEntityWithHistory subclass: #XpfUser
	instanceVariableNames: 'userName email projects password tasks lastAccess sharingPort wasOnline'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

!XpfUser commentStamp: 'mt 2/12/2009 12:24' prior: 0!
I am a system or application user. This could be an administrator, webmaster, developer or something like that.!

!XpfUser class methodsFor: 'description' stamp: 'topa 3/3/2009 22:37'!
collectionPropertyNames

	^ super collectionPropertyNames, #(tasks projects)! !

!XpfUser class methodsFor: 'authentication' stamp: 'topa 3/3/2009 22:37'!
hash: aString

	^ SecureHashAlgorithm new hashMessage: aString! !

!XpfUser class methodsFor: 'instance creation' stamp: 'SR 3/25/2009 18:01'!
newFrom: aDictionary

	^ self new
		userName: (aDictionary at: 'userName' ifAbsent: []);
		login: (aDictionary at: 'login');
		password: (aDictionary at: 'password' ifAbsent: [])! !

!XpfUser class methodsFor: 'default' stamp: 'topa 4/27/2009 14:07'!
onlineTimeout

	^ 10 seconds! !

!XpfUser class methodsFor: 'description' stamp: 'topa 4/27/2009 13:52'!
primitivePropertyNames

	^ super primitivePropertyNames, #(login userName email password isOnline sharingPort)! !

!XpfUser methodsFor: 'add-remove' stamp: 'cs 5/20/2009 14:07'!
addProject: aProject

	self pvtProjects 
		add: aProject
		ifPresent: [^ aProject].
	aProject addDeveloper: self.
	self 
		historyAddEntry: #added
		for: #projects
		to: aProject.
	^ aProject! !

!XpfUser methodsFor: 'add-remove' stamp: 'cs 5/20/2009 14:07'!
addTask: aTask

	self pvtTasks 
		add: aTask
		ifPresent: [^ aTask].
	aTask addDeveloper: self.
	self 
		historyAddEntry: #added
		for: #tasks
		to: aTask.
	^ aTask! !

!XpfUser methodsFor: 'authentication' stamp: 'SR 4/10/2009 12:18'!
authenticateBy: aString

	aString ifEmpty: [^ false]. "empty password is evil"
	^ self pvtPassword = (self class hash: aString)! !

!XpfUser methodsFor: 'deleting' stamp: 'mt 4/16/2009 13:16'!
delete

	self isAdmin ifTrue: [^ self].
	
	self projects do: [:project | 
		project historyAddEntry: #removed for: #developers to: self].
	self tasks do: [:task |
		task historyAddEntry: #removed for: #developers to: self].
	super delete.! !

!XpfUser methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:33'!
email

	^ email! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:27'!
email: anObject

	anObject = self email ifTrue: [^ self].
	self historyAddEntry: #changed for: #email from: self email to: anObject.
	email := anObject.! !

!XpfUser methodsFor: 'initialization' stamp: 'topa 4/22/2009 13:34'!
initialize

	super initialize.
	self
		pvtProjects: OrderedCollection new;
		pvtTasks: OrderedCollection new.! !

!XpfUser methodsFor: 'authentication' stamp: 'cs 6/2/2009 19:01'!
isAdmin

	^ self handler notNil and: [self = self handler root adminUser]! !

!XpfUser methodsFor: 'testing' stamp: 'cs 4/29/2009 12:00'!
isMethod: aMethodString allowedFor: aUser

	^ aMethodString = 'PUT'
		ifTrue: [self = aUser or: [aUser isAdmin]]
		ifFalse: [true]
! !

!XpfUser methodsFor: 'accessing' stamp: 'topa 4/27/2009 14:06'!
isOnline
	
	^ (TimeStamp now - self pvtLastAccess) <= self class onlineTimeout! !

!XpfUser methodsFor: 'accessing' stamp: 'topa 4/27/2009 14:15'!
isOnline: anObject
	"this is not writable"

	^ self! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/27/2009 15:35'!
lastAccess: aTimeStamp
	
	lastAccess := aTimeStamp.! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 3/4/2009 14:05'!
login

	^ email! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:27'!
login: anObject
	"You have to choose a valid email address as a login name."

	anObject = self email ifTrue: [^ self].
	self historyAddEntry: #changed for: #email from: self email to: anObject.
	email := anObject.! !

!XpfUser methodsFor: 'accessing' stamp: 'topa 4/7/2009 14:21'!
password
	"The password is never actually returned but compared to.
	This method is necessary for rest, too."
	
	^ nil! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:01'!
password: aString

	(aString notNil and: [(self class hash: aString) = password]) ifTrue: [^ self].
	self historyAddEntry: #changed for: #password.
	password := aString ifNotNil: [self class hash: aString].! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/15/2009 17:39'!
projects

	^ projects reject: [:project | project isDeleted]! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 5/6/2009 12:46'!
projectsWithDeleted

	^ projects! !

!XpfUser methodsFor: 'accessing' stamp: 'topa 4/27/2009 14:24'!
pvtLastAccess

	^ lastAccess ifNil: [TimeStamp fromString: '1-1-2000 11:55:00 am'].! !

!XpfUser methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:35'!
pvtPassword

	^ password! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/15/2009 17:49'!
pvtProjects

	^ projects! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/16/2009 11:25'!
pvtProjects: anObject

	projects := anObject.! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/16/2009 11:24'!
pvtTasks

	^ tasks! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/16/2009 11:24'!
pvtTasks: anObject

	tasks := anObject.! !

!XpfUser methodsFor: 'add-remove' stamp: 'cs 5/20/2009 14:08'!
removeProject: aProject

	self pvtProjects 
		remove: aProject
		ifAbsent: [^ aProject].
	self 
		historyAddEntry: #removed
		for: #projects
		to: aProject.
	aProject removeDeveloper: self.
	^ aProject! !

!XpfUser methodsFor: 'add-remove' stamp: 'cs 5/20/2009 14:08'!
removeTask: aTask

	self pvtTasks 
		remove: aTask
		ifAbsent: [^ aTask].
	self 
		historyAddEntry: #removed
		for: #tasks
		to: aTask.
	aTask removeDeveloper: self.
	^ aTask! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/29/2009 14:32'!
revision

	self isOnline ~= self wasOnline ifTrue: [
		self wasOnline: self isOnline.
		self incrementRevision].
	^ super revision! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/27/2009 16:53'!
sharingPort

	self isOnline ifFalse: [self sharingPort: nil].
	^ sharingPort! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/29/2009 14:32'!
sharingPort: anObject

	anObject = sharingPort ifTrue: [^ self].
	
	self incrementRevision.
	sharingPort := anObject.! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/15/2009 17:39'!
tasks

	^ tasks reject: [:task | task isDeleted]! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 5/6/2009 12:46'!
tasksWithDeleted

	^ tasks! !

!XpfUser methodsFor: 'updating' stamp: 'mt 5/7/2009 10:29'!
updateInterval
	"Tell the client how often a changes request for this entity should be sent."
	
	^ 1! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 3/4/2009 13:54'!
userName

	^ userName! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:28'!
userName: anObject

	self userName = anObject ifTrue: [^ self].
	self historyAddEntry: #changed for: #username from: self userName to: anObject.
	userName := anObject.! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/29/2009 14:35'!
wasOnline

	wasOnline ifNil: [wasOnline := false].
	^ wasOnline! !

!XpfUser methodsFor: 'accessing' stamp: 'mt 4/29/2009 14:33'!
wasOnline: anObject

	anObject = self wasOnline ifTrue: [^ self].
	
	wasOnline := anObject.
	self incrementRevision.! !

XpfEntityWithHistory subclass: #XpfUserStory
	instanceVariableNames: 'title priority timeEstimated tasks timeActual iteration positionX positionY color comments tests'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

!XpfUserStory commentStamp: 'topa 2/12/2009 14:06' prior: 0!
I am a user story in terms of Extreme Programming!

!XpfUserStory class methodsFor: 'description' stamp: 'topa 3/3/2009 22:40'!
collectionPropertyNames

	^ super collectionPropertyNames, #(tasks)! !

!XpfUserStory class methodsFor: 'instance creation' stamp: 'mt 4/26/2010 12:10:39.776'!
dictionaryClass

	^ Smalltalk at: #StringKeyValueDictionary ifAbsent: [Dictionary]! !

!XpfUserStory class methodsFor: 'instance creation' stamp: 'mt 11/7/2009 11:44'!
newFrom: aDictionary

	^ self new
		title: (aDictionary at: 'title' ifAbsent: []);
		priority: (aDictionary at: 'priority' ifAbsent: []);
		timeEstimated: (aDictionary at: 'timeEstimated' ifAbsent: []);
		timeActual: (aDictionary at: 'timeActual' ifAbsent: []);
		position: (aDictionary at: 'position' ifAbsent: [])! !

!XpfUserStory class methodsFor: 'description' stamp: 'topa 11/27/2009 17:48'!
primitivePropertyNames

	^ super primitivePropertyNames, 
		#(title priority timeActual timeEstimated positionX positionY color comments position tests)! !

!XpfUserStory class methodsFor: 'description' stamp: 'mt 4/16/2009 13:32'!
referencePropertyNames

	^ super referencePropertyNames, #(iteration)! !

!XpfUserStory methodsFor: 'add-remove' stamp: 'SR 3/25/2009 18:00'!
addNewTask: aPropertyDictionary

	^ self addTask: (XpfTask newFrom: aPropertyDictionary)! !

!XpfUserStory methodsFor: 'add-remove' stamp: 'cs 6/2/2009 13:31'!
addTask: aTask 

	| oldTaskRestUri |
	aTask id ifNil: [aTask id: aTask class incrementId].
	aTask userStory ifNotNil: 
		[oldTaskRestUri := aTask restUri.
		aTask userStory removeTask: aTask].
	aTask
		userStory: self;
		handler: self handler.
	(self pvtTasks includes: aTask) ifFalse: 
		[self pvtTasks add: aTask.
		self 
			historyAddEntry: #added
			for: #tasks
			to: aTask.
		oldTaskRestUri ifNotNil: 
			[XpfRestResourceHttpHandler 
				redirectRestUriFrom: oldTaskRestUri
				to: aTask restUri]].
	^ aTask! !

!XpfUserStory methodsFor: 'accessing' stamp: 'SR 3/18/2009 15:25'!
color

	^ color! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:28'!
color: anObject

	anObject = self color ifTrue: [^ self].
	self historyAddEntry: #changed for: #color from: self color to: anObject.
	color := anObject.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 4/30/2009 17:58'!
comments

	^ comments ifNil: [self comments: self class dictionaryClass new. comments]! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 4/30/2009 17:50'!
comments: anObject

	comments := anObject.! !

!XpfUserStory methodsFor: 'deleting' stamp: 'mt 4/16/2009 12:56'!
delete

	self tasks do: [:task | task delete].
	self iteration historyAddEntry: #removed for: #userStories to: self.
	super delete.! !

!XpfUserStory methodsFor: 'initialization' stamp: 'mt 4/15/2010 14:42:24.41'!
initialize

	super initialize.
	self 
		pvtTasks: OrderedCollection new;
		pvtTests: self class collectionClass new;
		positionX: 0.0;
		positionY: 0.0;
		color: 'white'.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/24/2009 17:40'!
iteration

	^ iteration! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:29'!
iteration: anObject

	self iteration = anObject ifTrue: [^ self].
	self historyAddEntry: #changed for: #iteration from: self iteration to: anObject.
	iteration := anObject.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 11/11/2009 12:56'!
position

	^ (self positionX	 ifNil: [0]) @ (self positionY ifNil: [0])! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 11/11/2009 12:51'!
position: aPoint

	aPoint = self position ifTrue: [^ self].
	self incrementRevision.
	positionX := aPoint ifNotNil: [aPoint x].
	positionY := aPoint ifNotNil: [aPoint y].! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/24/2009 17:41'!
positionX

	^ positionX! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/6/2009 17:03'!
positionX: anObject

	anObject = positionX ifTrue: [^ self].
	self incrementRevision.
	positionX := anObject.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/24/2009 17:41'!
positionY

	^ positionY! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/6/2009 17:04'!
positionY: anObject

	anObject = positionY ifTrue: [^ self].
	self incrementRevision.
	positionY := anObject.! !

!XpfUserStory methodsFor: 'printing' stamp: 'mt 2/25/2009 10:21'!
printString

	^ self title ifNil: ['No Title']! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:39'!
priority

	^ priority! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:29'!
priority: anObject

	anObject = self priority ifTrue: [^ self].
	self historyAddEntry: #changed for: #priority from: self priority to: anObject.
	priority := anObject.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/15/2009 16:41'!
pvtTasks

	^ tasks! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/15/2009 16:53'!
pvtTasks: anObject

	tasks := anObject.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/15/2010 14:41:48.9'!
pvtTests

	^ tests! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/15/2010 14:42:01.036'!
pvtTests: anObject

	tests := anObject.! !

!XpfUserStory methodsFor: 'private' stamp: 'cs 5/20/2009 14:09'!
removeTask: aTask 

	(self tasks noneSatisfy: [:t | t = aTask]) ifTrue: [^ aTask].
	self pvtTasks remove: aTask.
	self 
		historyAddEntry: #removed
		for: #tasks
		to: aTask.
	aTask userStory: nil.
	^ aTask! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/15/2009 16:41'!
tasks

	^ self pvtTasks reject: [:task | task isDeleted]! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 5/6/2009 12:46'!
tasksWithDeleted

	^ tasks! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/15/2010 14:42:10.175'!
tests

	^ self pvtTests! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 11/27/2009 17:50'!
tests: anObject

	anObject = self tests ifTrue: [^ self].
	self historyAddEntry: #changed for: #tests from: self tests to: anObject.
	tests := anObject.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:40'!
timeActual

	^ timeActual! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:29'!
timeActual: anObject

	anObject = self timeActual ifTrue: [^ self].
	self historyAddEntry: #changed for: #timeActual from: self timeActual to: anObject.
	timeActual := anObject.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:40'!
timeEstimated

	^ timeEstimated! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:29'!
timeEstimated: anObject

	anObject = self timeEstimated ifTrue: [^ self].
	self historyAddEntry: #changed for: #timeEstimated from: self timeEstimated to: anObject.
	timeEstimated := anObject.! !

!XpfUserStory methodsFor: 'accessing' stamp: 'topa 3/3/2009 22:40'!
title

	^ title! !

!XpfUserStory methodsFor: 'accessing' stamp: 'mt 4/16/2009 13:29'!
title: anObject

	anObject = self title ifTrue: [^ self].
	self historyAddEntry: #changed for: #title from: self title to: anObject.
	title := anObject.! !

!XpfUserStory methodsFor: 'updating' stamp: 'mt 5/7/2009 10:29'!
updateInterval
	"Tell the client how often a changes request for this entity should be sent."
	
	^ 2! !

XpfEntity subclass: #XpfHistoryEntry
	instanceVariableNames: 'entity timestamp user action property oldVal newVal oldRef newRef'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

!XpfHistoryEntry class methodsFor: 'add-remove' stamp: 'cs 6/2/2009 11:49'!
addNewProject: aDictionary

	^ self addProject: (self newFrom: aDictionary)! !

!XpfHistoryEntry class methodsFor: 'as yet unclassified' stamp: 'cs 4/17/2009 12:04'!
entity: entity action: action property: property from: from to: to

	^ self new
		entity: entity;
		action: action;
		property: property;
		old: from;
		new: to! !

!XpfHistoryEntry class methodsFor: 'description' stamp: 'cs 4/17/2009 11:57'!
primitivePropertyNames

	^ super primitivePropertyNames, #(timestamp action property oldVal newVal)! !

!XpfHistoryEntry class methodsFor: 'description' stamp: 'cs 4/17/2009 12:03'!
referencePropertyNames

	^ super referencePropertyNames, #(entity user oldRef newRef)! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:44'!
action

	^ action! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:44'!
action: anObject

	action := anObject.! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 12:01'!
entity

	^ entity! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 12:01'!
entity: anObject

	entity := anObject.! !

!XpfHistoryEntry methodsFor: 'as yet unclassified' stamp: 'topa 5/20/2009 16:32'!
initialize

	super initialize.
	self
		id: self class incrementId;
		timestamp: TimeStamp now printString;
		user: XpfCurrentUser value.! !

!XpfHistoryEntry methodsFor: 'as yet unclassified' stamp: 'cs 4/17/2009 13:04'!
isMethod: aMethodString allowedFor: aUser
	"returns whether the method is allowed for the specified user; may be overwritten in subclasses"

	^ self entity isMethod: aMethodString allowedFor: aUser! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:52'!
new: anObject

	anObject isXpfEntity
		ifTrue: [self newRef: anObject]
		ifFalse: [self newVal: anObject].! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:44'!
newRef

	^ newRef.! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:44'!
newRef: anObject

	newRef := anObject.! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:45'!
newVal

	^ newVal! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:45'!
newVal: anObject

	newVal := anObject.! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:51'!
old: anObject

	anObject isXpfEntity
		ifTrue: [self oldRef: anObject]
		ifFalse: [self oldVal: anObject].! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:45'!
oldRef

	^ oldRef! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:45'!
oldRef: anObject

	oldRef := anObject.! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:45'!
oldVal

	^ oldVal! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:45'!
oldVal: anObject

	oldVal := anObject.! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:45'!
property

	^ property! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:45'!
property: anObject

	property := anObject.! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:45'!
timestamp

	^ timestamp! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:45'!
timestamp: anObject

	timestamp := anObject.! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:46'!
user

	^ user! !

!XpfHistoryEntry methodsFor: 'accessing' stamp: 'cs 4/17/2009 11:46'!
user: anObject

	user := anObject.! !

RestEntity subclass: #XpfEvent
	instanceVariableNames: 'details project'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

!XpfEvent class methodsFor: 'as yet unclassified' stamp: 'FHH 7/9/2009 14:34'!
newFrom: aDictionary
	| instance dict |
	instance := self new.
	
	dict := aDictionary at: 'details' ifAbsent: [].
	instance details: dict.
	^ instance.! !

!XpfEvent class methodsFor: 'as yet unclassified' stamp: 'FHH 7/9/2009 14:34'!
primitivePropertyNames

	^ super primitivePropertyNames, #(details)! !

!XpfEvent methodsFor: 'deleting' stamp: 'FHH 7/9/2009 14:35'!
delete
	self project removeEvent: self.	! !

!XpfEvent methodsFor: 'accessing' stamp: 'FHH 7/9/2009 14:36'!
details
	^details ifNil: [ details := Dictionary new. ].! !

!XpfEvent methodsFor: 'accessing' stamp: 'FHH 7/9/2009 14:35'!
details: aDictionary
	details := aDictionary.! !

!XpfEvent methodsFor: 'accessing' stamp: 'FHH 7/9/2009 14:36'!
project
	^ project.! !

!XpfEvent methodsFor: 'accessing' stamp: 'FHH 7/9/2009 14:36'!
project: anXpfProject
	project := anXpfProject.! !

RestEntity subclass: #XpfRoot
	instanceVariableNames: 'projects users adminUser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Models'!

XpfRoot class
	instanceVariableNames: 'production'!

XpfRoot class
	instanceVariableNames: 'production'!

!XpfRoot class methodsFor: 'description' stamp: 'cs 5/28/2009 15:29'!
collectionPropertyNames

	^ #(projects users)! !

!XpfRoot class methodsFor: 'accessing' stamp: 'cs 6/3/2009 14:13'!
production

	^ production ifNil: [production := self new]! !

!XpfRoot class methodsFor: 'accessing' stamp: 'cs 6/3/2009 14:11'!
production: anObject

	production := anObject.! !

!XpfRoot class methodsFor: 'reset' stamp: 'mt 4/15/2010 15:23:07.104'!
reset
	"Warning: All data will be lost!!"

	(UserDialogBoxMorph
		confirm: 'All data will be lost!!'
		title: 'Do you really want to reset the database?')
		ifTrue: [self production: nil].! !

!XpfRoot methodsFor: 'add-remove' stamp: 'cs 6/2/2009 11:58'!
addNewProject: aDictionary

	^ self addProject: (XpfProject newFrom: aDictionary)! !

!XpfRoot methodsFor: 'add-remove' stamp: 'cs 6/2/2009 11:58'!
addNewUser: aDictionary

	^ self addUser: (XpfUser newFrom: aDictionary)! !

!XpfRoot methodsFor: 'add-remove' stamp: 'cs 6/2/2009 15:37'!
addProject: aProject

	(self pvtProjects includes: aProject) ifTrue: [^ aProject].
	aProject
		id: XpfProject incrementId;
		handler: self handler.
	^ self pvtProjects add: aProject! !

!XpfRoot methodsFor: 'add-remove' stamp: 'cs 6/2/2009 15:38'!
addUser: aUser

	(self pvtUsers includes: aUser) ifTrue: [^ aUser].
	aUser
		id: XpfUser incrementId;
		handler: self handler.
	^ self pvtUsers add: aUser! !

!XpfRoot methodsFor: 'accessing' stamp: 'cs 6/2/2009 11:53'!
adminUser

	^ adminUser! !

!XpfRoot methodsFor: 'accessing' stamp: 'cs 6/2/2009 11:53'!
adminUser: anObject

	adminUser := anObject.! !

!XpfRoot methodsFor: 'authentication' stamp: 'cs 6/2/2009 11:59'!
authenticate: aLogin by: aPassword
	
	^ self users 
		detect: [ :aUser| (aUser login = aLogin) and: [aUser authenticateBy: aPassword]]
		ifNone: []! !

!XpfRoot methodsFor: 'authentication' stamp: 'cs 6/2/2009 11:57'!
findUserByLogin: aLoginString

	^ self users detect: [:user | user login = aLoginString]! !

!XpfRoot methodsFor: 'initialization' stamp: 'mt 10/30/2009 12:46'!
initialize

	super initialize.
	self
		pvtProjects: Set new;
		pvtUsers: Set new;
		adminUser: (self addNewUser: (Dictionary newFrom: {
			'userName' -> 'Administrator'. 
			'login' -> 'admin'. 
			'password' -> 'nimda'})).! !

!XpfRoot methodsFor: 'accessing' stamp: 'cs 6/2/2009 15:33'!
projects

	^ self pvtProjects reject: [:project | project isDeleted]! !

!XpfRoot methodsFor: 'accessing' stamp: 'cs 6/2/2009 15:39'!
projectsWithDeleted

	^ self pvtProjects copy! !

!XpfRoot methodsFor: 'private' stamp: 'cs 6/2/2009 15:32'!
pvtProjects

	^ projects! !

!XpfRoot methodsFor: 'private' stamp: 'cs 6/2/2009 15:36'!
pvtProjects: aCollection

	projects := aCollection.! !

!XpfRoot methodsFor: 'private' stamp: 'cs 6/2/2009 15:36'!
pvtUsers

	^ users! !

!XpfRoot methodsFor: 'private' stamp: 'cs 6/2/2009 15:36'!
pvtUsers: aCollection

	users := aCollection.! !

!XpfRoot methodsFor: 'add-remove' stamp: 'cs 6/2/2009 15:39'!
removeProject: aProject
	
	^ self pvtProjects remove: aProject! !

!XpfRoot methodsFor: 'add-remove' stamp: 'cs 6/2/2009 15:39'!
removeUser: aUser
	
	^ self pvtUsers remove: aUser! !

!XpfRoot methodsFor: 'accessing' stamp: 'cs 6/2/2009 15:37'!
users

	^ self pvtUsers reject: [:user | user isDeleted]! !

!XpfRoot methodsFor: 'accessing' stamp: 'cs 6/2/2009 15:38'!
usersWithDeleted

	^ self pvtUsers copy! !

RestNeedsAuthentication subclass: #XpfRestAuthentication
	instanceVariableNames: 'root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest'!

!XpfRestAuthentication class methodsFor: 'constants' stamp: 'cs 6/3/2009 14:05'!
realm

	^ 'XPForums'! !

!XpfRestAuthentication class methodsFor: 'instance creation' stamp: 'cs 6/3/2009 14:19'!
with: root

	^ self new root: root! !

!XpfRestAuthentication methodsFor: 'users' stamp: 'cs 6/3/2009 14:48'!
doWithCurrentUser: aBlock request: anHTTPRequest

	XpfCurrentUser 
		use: (self userFromRequest: anHTTPRequest)
		during: [^ aBlock value].! !

!XpfRestAuthentication methodsFor: 'authentication' stamp: 'cs 6/3/2009 14:50'!
requestAllowed: anHTTPRequest for: anObject

	| user |
	(user := self userFromRequest: anHTTPRequest) isAdmin ifFalse: [
		(anObject isXpfEntity and: 
		[(anObject isMethod: anHTTPRequest class methodName allowedFor: user) not])
			ifTrue: [RestNotAllowedError signal: 'Access forbidden for this resource']].! !

!XpfRestAuthentication methodsFor: 'accessing' stamp: 'cs 6/3/2009 14:14'!
root

	^ root ifNil: [root := XpfRoot production]! !

!XpfRestAuthentication methodsFor: 'accessing' stamp: 'cs 6/3/2009 14:14'!
root: anObject

	root := anObject.! !

!XpfRestAuthentication methodsFor: 'users' stamp: 'cs 6/3/2009 14:50'!
userFromRequest: anHTTPRequest

	| headerField |
	headerField := anHTTPRequest headers
		fieldNamed: 'AUTHORIZATION' 
		ifNone: [RestNotAllowedError signal: 'No authorization information supplied.'].
		
	^ (self root authenticate: headerField userid by: headerField password)
		ifNil: [RestNotAllowedError signal: 'Username and password does not match.']! !

!XpfRestAuthentication methodsFor: 'authentication' stamp: 'cs 6/3/2009 15:04'!
username: username password: password

	^ (self root authenticate: username by: password) notNil! !

RestGetRequestProcessor subclass: #XpfNewestEventsProcessor
	instanceVariableNames: 'since'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest'!

!XpfNewestEventsProcessor methodsFor: 'as yet unclassified' stamp: 'FHH 7/9/2009 11:11'!
referenceTime
	^ referenceTime ifNil: [ referenceTime := DateAndTime now. ].! !

!XpfNewestEventsProcessor methodsFor: 'as yet unclassified' stamp: 'FHH 7/9/2009 11:12'!
referenceTime: aDateAndTime
	referenceTime := aDateAndTime.! !

!XpfNewestEventsProcessor methodsFor: 'as yet unclassified' stamp: 'FHH 7/9/2009 20:51'!
responseBodyForRoot: root components: pathComponents
	"Do recursive entity resolving. Start with root."
		
	| entity |
	entity := self resolveMethod: root components: pathComponents.
	^ Json render: (entity events select: [:event | 
		((event details isKindOf: Dictionary) ifTrue: [
			(DateAndTime fromString: (event details at: 'dateAndTime' ifAbsent: [])) >= self referenceTime.
		] ifFalse: [
			false.
		]).
	]).! !

RestHttpHandler subclass: #XpfRestFeedbackHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest'!

!XpfRestFeedbackHttpHandler class methodsFor: 'handling' stamp: 'cs 5/19/2009 15:31'!
canBeAutostarted

	^ true! !

!XpfRestFeedbackHttpHandler class methodsFor: 'handling' stamp: 'cs 5/20/2009 16:43'!
pathPattern

	^ super pathPattern , 'feedback'! !

!XpfRestFeedbackHttpHandler methodsFor: 'request processing' stamp: 'SR 4/15/2009 14:38'!
process: anHTTPRequest
	"Handles an http request. Should answer with an http response."
	
	^ HTTPResponse ok entity:''! !

RestHttpHandler subclass: #XpfRestInfoHttpHandler
	instanceVariableNames: 'root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest'!

!XpfRestInfoHttpHandler class methodsFor: 'authentication' stamp: 'cs 6/3/2009 14:05'!
authentication

	^ XpfRestAuthentication new! !

!XpfRestInfoHttpHandler class methodsFor: 'testing' stamp: 'cs 5/19/2009 15:31'!
canBeAutostarted

	^ true! !

!XpfRestInfoHttpHandler class methodsFor: 'handling' stamp: 'cs 5/20/2009 16:43'!
pathPattern

	^ super pathPattern , 'info'! !

!XpfRestInfoHttpHandler methodsFor: 'request processing' stamp: 'cs 6/3/2009 15:40'!
process: anHTTPRequest
	"Handles an http request. Should answer with an http response."
	
	| user |
	user := self authentication userFromRequest: anHTTPRequest.
	^ HTTPResponse ok entity: 
		((XpfRestResourceHttpHandler uriMapping at: user class) value: user)! !

RestResourceHttpHandler subclass: #XpfRestResourceHttpHandler
	instanceVariableNames: ''
	classVariableNames: 'ObsoleteRestUriTable'
	poolDictionaries: ''
	category: 'Xpf-Server-Rest'!

XpfRestResourceHttpHandler subclass: #XpfNewestEventsHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest'!

!XpfNewestEventsHttpHandler class methodsFor: 'as yet unclassified' stamp: 'test 5/26/2009 17:27'!
pathPattern

	^ 'xpaChanges'.! !

!XpfNewestEventsHttpHandler methodsFor: 'as yet unclassified' stamp: 'FHH 7/9/2009 18:33'!
postDataFor: anHTTPRequest

	 | data |
	anHTTPRequest class category = 'Hyper' ifFalse: [anHTTPRequest ensureFullRead].
	data := Json readFrom: anHTTPRequest entityBody value readStream.
	^ data! !

!XpfNewestEventsHttpHandler methodsFor: 'as yet unclassified' stamp: 'topa 08/24/2009 14:29'!
process: anHTTPRequest
	^ anHTTPRequest isPost
		ifFalse: [^ HTTPResponse badRequest]
		ifTrue:
		[
			[
				| postData |
				postData := (self postDataFor: anHTTPRequest).
				(XpfNewestEventsProcessor newFrom: anHTTPRequest handler: self)
					referenceTime: (DateAndTime fromString: postData second);
					process: self root components: (postData first findTokens: '/').
			] on: Error do: [:e| ^ HTTPResponse badRequest entity: e description]
		]! !

XpfRestResourceHttpHandler subclass: #XpfRestChangesHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest'!

!XpfRestChangesHttpHandler class methodsFor: 'as yet unclassified' stamp: 'topa 5/20/2009 19:18'!
pathPattern

	^ 'changes'! !

!XpfRestChangesHttpHandler methodsFor: 'request processing' stamp: 'mt 4/21/2009 13:11'!
findChanges: anHTTPRequest

	| wrapper result |
	result := OrderedCollection new.
	(self postDataFor: anHTTPRequest) keysAndValuesDo:	[ :restUri :revision | 
		wrapper := XpfEntityRevisionWrapper new
			restUri: restUri;
			clientRevision: revision;
			resolve: anHTTPRequest handler: self.
			wrapper needsUpdate ifTrue: [ result add: wrapper restUri ]].
	^ self responseFor: result! !

!XpfRestChangesHttpHandler methodsFor: 'request processing' stamp: 'mig 4/1/2009 15:53'!
postDataFor: anHTTPRequest

	 | data |
	anHTTPRequest class category = 'Hyper' ifFalse: [anHTTPRequest ensureFullRead].
	data := Json readFrom: anHTTPRequest entityBody value readStream.
	^ data! !

!XpfRestChangesHttpHandler methodsFor: 'request processing' stamp: 'topa 5/20/2009 16:42'!
process: anHTTPRequest

	^ anHTTPRequest isPost 
		ifFalse: [HTTPResponse badRequest]
		ifTrue: [
			self setOnlineFromRequest: anHTTPRequest.
			self findChanges: anHTTPRequest]! !

!XpfRestChangesHttpHandler methodsFor: 'request processing' stamp: 'mig 4/1/2009 15:46'!
responseFor: aCollection

	^ HTTPResponse ok entity: (Json render: aCollection)! !

!XpfRestChangesHttpHandler methodsFor: 'request processing' stamp: 'cs 6/3/2009 14:41'!
setOnlineFromRequest: anHTTPRequest
	
	(self authentication userFromRequest: anHTTPRequest)
		ifNotNilDo: [:user| user
			lastAccess: TimeStamp now;
			wasOnline: true].! !

!XpfRestResourceHttpHandler class methodsFor: 'authentication' stamp: 'cs 6/3/2009 15:04'!
authentication

	^ XpfRestAuthentication new! !

!XpfRestResourceHttpHandler class methodsFor: 'testing' stamp: 'topa 5/20/2009 16:38'!
canBeAutostarted

	^ true! !

!XpfRestResourceHttpHandler class methodsFor: 'redirecting' stamp: 'mt 4/21/2009 18:23'!
lookupObsoleteRestUri: aString

	ObsoleteRestUriTable ifNil: [ObsoleteRestUriTable := Dictionary new].
	^   ObsoleteRestUriTable at: aString ifAbsent: [].! !

!XpfRestResourceHttpHandler class methodsFor: 'redirecting' stamp: 'cs 5/22/2009 13:55'!
redirectRestUriFrom: sourceUriString to: targetUriString

	ObsoleteRestUriTable ifNil: [ObsoleteRestUriTable := Dictionary new].
	ObsoleteRestUriTable
	 	at: self pathPattern, sourceUriString put: self pathPattern, targetUriString;
		removeKey: self pathPattern, targetUriString ifAbsent: [].! !

!XpfRestResourceHttpHandler class methodsFor: 'resources' stamp: 'cs 6/3/2009 15:28'!
rootResource

	^ XpfRoot production! !

!XpfRestResourceHttpHandler class methodsFor: 'resources' stamp: 'mt 4/15/2010 15:43:09.789'!
uriMapping

	^ Dictionary newFrom:{
		XpfUser -> [:user| '/usersWithDeleted/{1}' format: {user id}].		
		XpfProject -> [:project| '/projectsWithDeleted/{1}' format: {project id}].
		XpfFutureIterationForProject -> [:iter | '{1}/openStories/{2}' format: {iter project restUri. iter id}].
		XpfRelease -> [:release | '{1}/releasesWithDeleted/{2}' format: {release project restUri. release id}].
		XpfIteration -> [:iter | '{1}/iterationsWithDeleted/{2}' format: {iter theRelease restUri. iter id}].
		XpfFutureIteration -> [:iter | '{1}/iterationsWithDeleted/{2}' format: {iter theRelease restUri. iter id}].
		XpfUserStory -> [:us | '{1}/userStoriesWithDeleted/{2}' format: {us iteration restUri. us id}].
		XpfTask -> [:task | '{1}/tasksWithDeleted/{2}' format: {task userStory restUri. task id}].		
		XpfHistoryEntry -> [:hist | '{1}/history/{2}' format: {hist entity restUri. hist id}].
		XpfEvent -> [:event | '{1}/events/{2}' format: {event project restUri. event id}]}! !

!XpfRestResourceHttpHandler class methodsFor: 'instance creation' stamp: 'cs 6/3/2009 14:21'!
with: root

	| instance |
	instance := super with: root.
	instance authentication root: root.
	^ instance! !

!XpfRestResourceHttpHandler methodsFor: 'request processing' stamp: 'cs 6/3/2009 14:47'!
process: anHTTPRequest

	^ self authentication
		doWithCurrentUser: [
			[super process: anHTTPRequest]
				on: RestResolveError
				do: [:error | self processForNotFound: anHTTPRequest previousResult: error]]
		request: anHTTPRequest! !

!XpfRestResourceHttpHandler methodsFor: 'request processing' stamp: 'topa 5/25/2009 19:03'!
processForNotFound: anHTTPRequest previousResult: result

	^ (self class lookupObsoleteRestUri: anHTTPRequest uriString)
			ifNil: [result signal]
			ifNotNilDo: [:restUri |
				HTTPResponse redirectLink addHeaderName: 'location' value: restUri; yourself]! !

!XpfRestResourceHttpHandler methodsFor: 'authentication' stamp: 'mt 4/15/2010 15:56:36.006'!
selectorAllowed: aSelector for: anXpfEntity

	^ (super selectorAllowed: aSelector for: anXpfEntity)
		or:[(anXpfEntity class collectionPropertyNames collect: [:name | 
			(name asString, 'WithDeleted') asSymbol]) includes: aSelector]
		or: [(anXpfEntity class referencePropertyNames includes: #futureIteration)
			and: [aSelector = #openStories]]! !

!Object methodsFor: '*xpf-server-models' stamp: 'mfp 3/6/2009 11:45'!
isXpfEntity
	^ false! !

Object subclass: #XpfEntityRevisionWrapper
	instanceVariableNames: 'clientRevision entity restUri isValid'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest'!

!XpfEntityRevisionWrapper methodsFor: 'accessing' stamp: 'mig 4/1/2009 17:17'!
clientRevision

	^ clientRevision! !

!XpfEntityRevisionWrapper methodsFor: 'accessing' stamp: 'mig 4/1/2009 17:17'!
clientRevision: anObject

	clientRevision := anObject.! !

!XpfEntityRevisionWrapper methodsFor: 'accessing' stamp: 'mig 4/1/2009 17:17'!
entity

	^ entity! !

!XpfEntityRevisionWrapper methodsFor: 'accessing' stamp: 'mig 4/1/2009 17:17'!
entity: anObject

	entity := anObject.! !

!XpfEntityRevisionWrapper methodsFor: 'initialization' stamp: 'mt 4/21/2009 13:18'!
initialize

	super initialize.
	self isValid: true.! !

!XpfEntityRevisionWrapper methodsFor: 'accessing' stamp: 'mt 4/21/2009 13:15'!
isValid

	^ isValid! !

!XpfEntityRevisionWrapper methodsFor: 'accessing' stamp: 'mt 4/21/2009 13:15'!
isValid: anObject 

	isValid := anObject.! !

!XpfEntityRevisionWrapper methodsFor: 'testing' stamp: 'cs 5/15/2009 14:24'!
needsUpdate

	^ self entity isNil
		or: [self isValid not
		or: [self entity revision > self clientRevision]]! !

!XpfEntityRevisionWrapper methodsFor: 'resolving' stamp: 'mt 4/21/2009 15:30'!
resolve: anHTTPRequest handler: handler

	[self entity:
		(self resolveUri: self restUri request: anHTTPRequest handler: handler)]
		on: RestError
		do: [:e | self isValid: false. ^ self].
	self entity isCollection ifTrue:
		[self entity:
			(self resolveUri: self shortRestUri request: anHTTPRequest handler: handler)]! !

!XpfEntityRevisionWrapper methodsFor: 'request processing' stamp: 'mig 4/1/2009 18:08'!
resolveUri: aString request: anHTTPRequest handler: handler

	^ aString isEmpty
		ifTrue: []
		ifFalse: [RestRequestProcessor new
			handler: handler;
			request: anHTTPRequest;
			resolveUri: aString]
! !

!XpfEntityRevisionWrapper methodsFor: 'accessing' stamp: 'mig 4/1/2009 17:17'!
restUri

	^ restUri! !

!XpfEntityRevisionWrapper methodsFor: 'accessing' stamp: 'mig 4/1/2009 17:18'!
restUri: anObject

	restUri := anObject.! !

!XpfEntityRevisionWrapper methodsFor: 'resolving' stamp: 'mig 4/1/2009 17:55'!
shortRestUri

	| result |
	result := String new.
	(self restUri findTokens: $/) allButLast
		do: [:each | result := result, each]
		separatedBy: [result := result, '/'].
	^ result! !

RestDynamicVariable subclass: #XpfCurrentUser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xpf-Server-Rest'!
